"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _numberFormat = _interopRequireDefault(require("./number-format"));

var _en = _interopRequireDefault(require("./plurals/en"));

var _ru = _interopRequireDefault(require("./plurals/ru"));

var _es = _interopRequireDefault(require("./plurals/es"));

var _fr = _interopRequireDefault(require("./plurals/fr"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const canUseConsole = typeof console == 'object';
const pluralMap = {};

function logError() {
  if (canUseConsole) {
    const list = Array.prototype.slice.apply(arguments);
    list.splice(0, 0, '#translate.plurals:');
    console.error.apply(console, list);
  }
}

function addPlural(plu) {
  if (pluralMap[plu.lang]) {
    logError(`trying register already defined plural - ${plu.lang}`);
  } else {
    pluralMap[plu.lang] = plu;
  }
}

function plural(lang, fields, data) {
  const plu = pluralMap[lang];
  let ret = fields.key || '';
  let asMap = false;

  if (typeof data === 'object') {
    asMap = true;
  }

  for (const key in fields) {
    if (key != 'key') {
      const num = asMap ? data[key] : data;
      const pluArray = fields[key];
      const index = plu.getIndex(num);
      let pluStr = pluArray ? pluArray[index] : '';
      const numFormatted = (0, _numberFormat.default)(lang, num);
      pluStr = pluStr.replace('{#}', numFormatted);
      ret = ret.replace(`{#${key}}`, pluStr);
    }
  }

  if (asMap) {
    for (let key in data) {
      if (!fields.hasOwnProperty(key)) {
        const val = data[key];
        ret = ret.replace(`{#${key}}`, val);
      }
    }
  }

  return ret;
}

plural.addPlural = function (plu) {
  if (Array.isArray(plu)) {
    plu.forEach(function (item) {
      addPlural(item);
    });
  } else {
    addPlural(plu);
  }
}; // default plurals


plural.addPlural([_en.default, _ru.default, _es.default, _fr.default]);
var _default = plural;
exports.default = _default;